#include "burp.h"
#include "prog.h"
#include "msg.h"
#include "lock.h"

static void usage(void)
{
	logp("usage: %s <options>\n", progname());
}

#if defined(HAVE_WIN32)
#define main BurpMain
#endif
int main (int argc, char *argv[])
{
	int ret=0;
	int option=0;
	int forking=1;
	struct config conf;
	int forceoverwrite=0;
	enum action act=ACTION_LIST;
	const char *backup=NULL;
	const char *restoreprefix=NULL;
	const char *regex=NULL;
#ifdef HAVE_WIN32
	const char *configfile="C:/Program Files/Burp/burp.conf";
#else
	const char *configfile="/etc/burp/burp.conf";
#endif

	init_log(argv[0]);

	while((option=getopt(argc, argv, "a:b:d:c:fnr:h?"))!=-1)
	{
		switch(option)
		{
			case 'a':
				if(!strncmp(optarg, "backup", 1))
					act=ACTION_BACKUP;
				else if(!strncmp(optarg, "restore", 1))
					act=ACTION_RESTORE;
				else if(!strncmp(optarg, "list", 1))
					act=ACTION_LIST;
				else if(!strncmp(optarg, "List", 1))
					act=ACTION_LONG_LIST;
				else
				{
					usage();
					return 1;
				}
				break;
			case 'b':
				backup=optarg;
				break;
			case 'c':
				configfile=optarg;
				break;
			case 'd':
				restoreprefix=optarg;
				break;
			case 'f':
				forceoverwrite=1;
				break;
			case 'n':
				forking=0;
				break;
			case 'r':
				regex=optarg;
				break;
			case 'h':
			case '?':
			default:
				usage();
				return 1;
				break;
		}
	}
	if(optind<argc)
	{
		usage();
		return 1;
	}

	if(act==ACTION_RESTORE && !backup)
	{
		logp("No backup specified. Using the most recent.\n");
		backup="0";
	}

	init_config(&conf);
	if(load_config(configfile, &conf, 1)) return 1;

	// Allow listing when another process is backing up / restoring.
	if(conf.mode==MODE_CLIENT && (act==ACTION_LIST || act==ACTION_LONG_LIST))
	{
		//logp("list mode\n");
		//if(backup) logp("backup: %s\n", backup);
	}
	else if(get_lock(conf.lockfile))
	{
		logp("Could not get lockfile.\n");
		logp("Another process is probably running,\n");
		logp("or you do not have permissions to write to %s.\n",
			conf.lockfile);
		return 1;
	}

	if(conf.mode==MODE_SERVER)
	{
#ifdef HAVE_WIN32
		logp("Sorry, server mode is not implemented for Windows.\n");
#else
		ret=server(&conf, forking);
#endif
	}
	else
	{
		ret=client(&conf, act, backup,
			restoreprefix, regex, forceoverwrite);
	}

	unlink(conf.lockfile);
	free_config(&conf);
	return ret;
}
